import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class DLASim{
	JFrame frame;//メインウィンドウ
    JPanel panel;

	BufferedImage image;//バッファ領域（ダブルバッファリング用）

	Graphics fg;//描画領域用描画オブジェクト（グラフィックスコンテキスト）
	Graphics bg;//ダブルバッファリング用描画オブジェクト（グラフィックスコンテキスト）

	int screenW;//ウィンドウの幅
	int screenH;//ウィンドウの高さ

	final int N=256;//フィールドサイズ
	final int N2=N/2;

	Random rand; //擬似乱数発生器
	int particleNum=5000;//粒子数
	int particleDiameter=2;//粒子の直径
	int[] px=new int[particleNum];//粒子のX座標
	int[] py=new int[particleNum];//粒子のY座標
	boolean[] fix=new boolean[particleNum];//粒子の状態（０：浮遊状態、１：固定状態

	//画面の初期化処理
	public void initScreen() {
		//画面サイズの設定
		screenW=N*particleDiameter;
		screenH=N*particleDiameter;

		//フレームの設定
		frame=new JFrame();
		frame.setTitle("DLA Simulation");
		frame.setBounds(200,200,screenW,screenH);
		frame.setResizable(false);//ウィンドウサイズを変更不可に設定
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		Container cont=frame.getContentPane();
		cont.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
		panel=new JPanel();
		panel.setPreferredSize(new Dimension(screenW,screenH));
		cont.add(panel);

		frame.pack();
		frame.setVisible(true);

		//描画領域のサイズ調整
        panel.setPreferredSize(new Dimension(screenW,screenH));
		frame.pack();

		//フレームに描画する準備
		image=(BufferedImage)panel.createImage(screenW, screenH);
		fg=panel.getGraphics();
		bg=image.getGraphics();
	}

	//xの値をlowerからupperの範囲に制限する
	int clipInt(int x,int lower, int upper) {
		return (x<lower?lower:(x>upper?upper:x));
	}

	//初期条件の設定
	void setInitialCondition(){
		//擬似乱数発生器の初期化
		rand=new Random();

		//初期パーティクルの設定
		//粒子をランダムに配置
		for(int i=1;i<particleNum;i++) {
			px[i]=rand.nextInt(N);
			py[i]=rand.nextInt(N);
			fix[i]=false;
		}

		//画面中央に固定粒子を配置
		px[0]=N2;
		py[0]=N2;
		fix[0]=true;

		//初期表示
		bg.setColor(Color.black);
		bg.fillRect(0, 0, screenW,screenH);
		fg.drawImage(image, 0, 0, panel);
	}

	//粒子の表示
	public void drawParticle() {
		for(int k=0;k<particleNum;k++) {

			bg.setColor(Color.black);
			bg.fillOval(px[k]*particleDiameter, py[k]*particleDiameter, particleDiameter*2, particleDiameter*2);

			double theta=Math.random()*2*Math.PI;
			int x=(int)(px[k]+Math.cos(theta)+0.5);
			int y=(int)(py[k]+Math.sin(theta)+0.5);

			for(int i=0;i<particleNum;i++) {
				if(!fix[i])continue;
				int dist=(px[i]-px[k])*(px[i]-px[k])+(py[i]-py[k])*(py[i]-py[k]);
				if(dist<=2) {
					fix[k]=true;
				}
			}

			if(!fix[k]) {
				px[k]=clipInt(x,0,N-1);
				py[k]=clipInt(y,0,N-1);
			}

			//Vを表示
			if(!fix[k]) {
				bg.setColor(Color.white);
				bg.fillOval(px[k]*particleDiameter, py[k]*particleDiameter, particleDiameter*2, particleDiameter*2);
			}else {
				bg.setColor(Color.red);
				bg.fillOval(px[k]*particleDiameter, py[k]*particleDiameter, particleDiameter*2, particleDiameter*2);
			}
		}
		//バッファ領域のイメージを描画領域へ転送
		fg.drawImage(image, 0, 0, panel);
	}

	public DLASim(){

		//画面の初期化
		initScreen();
		//初期条件の設定
		setInitialCondition();
		//粒子の描画
		drawParticle();

		while(true) {
			drawParticle();
		}
	}

	public static void main(String[] args){
		new DLASim();
	}
}
